#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "config.h"
#include "data.h"
#include "arpreader.h"

ARPReader::ARPReader(Config* conf)
{
  Conf = conf;
}

int ARPReader::parseLine(char *s, unsigned *ip, unsigned *flags,
                         char **device)
{
  char *t;

  s = skip_whitespace(s);       //mezery na zacatku radku
  if (!*s)
    return 0;
  t = skip_token(s);            //1 sloupec ip
  if (!*t)
    return 0;
  *t = 0;
  *ip = parse_ip(s);
  if (!*ip)
    return 0;

  s = skip_whitespace(t + 1);
  if (!*s)
    return 0;
  s = skip_token(s);            //2 sloupec hwtype
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;
  t = skip_token(s);            //3 sloupec flags
  if (!*t)
    return 0;
  *t = 0;
  if (!sscanf(s, "%x", flags))
    return 0;

  s = skip_whitespace(t + 1);
  if (!*s)
    return 0;
  s = skip_token(s);            //4 sloupec hwadress
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;
  s = skip_token(s);            //5 sloupec mask
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;
  t = skip_token(s);            //6 sloupec device
  if (!*t)
    return 0;
  *t = 0;
  *device = s;

  return 1;
}

int ARPReader::isDeviceActive(char *device)
{
  for (int i = 0; i < Conf->DeviceNum; i++)
  {
    if (strcmp(device, Conf->DeviceList[i]) == 0)
      return 1;
  }
  return 0;
}

int ARPReader::Read(TrafficData * data, char* filename)
{
  char *s = NULL;
  char *device = NULL;
  size_t len = 0;
  unsigned flags;
  FILE *f = fopen(filename, "rt");
  if (!f)
  {
    fprintf(stderr, "Nelze otevrit %s: %s\n", filename,
            strerror(errno));
    return -1;
  }

  unsigned ip;
  while (getline(&s, &len, f) != -1)
  {
    if (parseLine(s, &ip, &flags, &device))
    {
      if (flags & 0x2 && isDeviceActive(device))
        data->addFlags(ip, F_ARP);
    }
  }

  if (s)
    free(s);

  fclose(f);
  return 0;
}
