#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "config.h"
#include "data.h"
#include "cgioutput.h"

char InsertionMark[] = "***INSERT_TABLE***";
char *ClassIp = "Ip";
char *ClassUpload = "Upload";
char *ClassDownload = "Download";
char *ClassSelected = "Selected";

char *Suffix[] = { "B", "kB", "MB", "GB", "TB", 0 };

bool CGIOutput::PrintPage(Config * conf, TrafficTableIterator * it,
                         int selectedColumn)
{
  char *s = NULL;
  size_t len = 0;
  FILE *f = fopen(conf->HTMLTemplate, "rt");
  if (!f)
  {
    fprintf(stderr, "Nelze otevrit %s: %s\n", conf->HTMLTemplate,
            strerror(errno));
    return false;
  }

  printf("Content-type: text/html\n");
  if (conf->WWWRefresh)
    printf("Refresh: %d\n", conf->WWWRefresh);
  printf("Pragma: no-cache\n");

/*      char exp[MAX_LINELEN];
        time_t exp_time = time(NULL) + Refresh;
        strftime(exp, MAX_LINELEN, "%a, %d %b %Y %H:%M:%S GMT", gmtime(&exp_time));
        printf("Expires: %s\n",exp);*/

  printf("\n");

  while (getline(&s, &len, f) != -1)
  {
    if (strncasecmp(s, InsertionMark, strlen(InsertionMark)) == 0)
      PrintTable(it, conf, selectedColumn);
    else
      printf("%s", s);
  }

  fclose(f);
  if (s)
    free(s);

  return true;
}

bool CGIOutput::PrintTable(TrafficTableIterator * it, Config * conf,
                          unsigned int selectedColumn)
{
  if (!it)
  {
    fprintf(stderr, "No data.\n");
    return false;
  }

  char *str_ip;
  do
  {
    str_ip = string_ip(it->getIP());
    printf("<tr><td align=\"center\" class=\"%s\">",
           (selectedColumn == 0 ? ClassSelected : ClassIp));
    printf("<a href=\"%s?ip=%s\">%s</a></td>", conf->GraphScriptURL, str_ip,
           str_ip);

    for (unsigned int i = 1; i < it->getCounterNumber(); i++)
    {
      printf("<td align=\"right\" class=\"%s\">%s</td>",
             (selectedColumn == ((i * 2) - 1) ? ClassSelected : ClassUpload),
             NiceDataPrint(it->getUpload(i)));
      printf("<td align=\"right\" class=\"%s\">%s</td>",
             (selectedColumn == ((i * 2)) ? ClassSelected : ClassDownload),
             NiceDataPrint(it->getDownload(i)));
    }
    printf("</tr>\n");
  }
  while (it->Next());

  return true;
}

char *CGIOutput::NiceDataPrint(unsigned long long bytes)
{
  static char ret[MAX_LINELEN];

  int i = 0;
  double dd = bytes;
  while (dd > 1024)
  {
    i++;
    if (!Suffix[i])
    {
      i--;
      break;
    }
    dd /= 1024;
  }

  if (i)
    sprintf(ret, "%3.02f %s", dd, Suffix[i]);
  else
    sprintf(ret, "%llu&nbsp;&nbsp;%s", bytes, Suffix[i]);

  return ret;
}
