#include <time.h>

#define F_IPT_IN 1
#define F_IPT_OUT 2
#define F_ARP 4
#define F_LBCFREE 8
#define F_LIST 16

class TrafficDataIterator;
class TrafficTableIterator;

class TrafficData
{
public:
  TrafficData();
  ~TrafficData();

  bool setTimeStamp(const char *time);
  bool addIP(unsigned int ip);
  bool setUpload(unsigned int ip, const char *counter, const char *byte_str);
  bool setUpload(unsigned int ip, int counter, unsigned long long bytes);
  bool setDownload(unsigned int ip, const char *counter, const char *byte_str);
  bool setDownload(unsigned int ip, int counter, unsigned long long bytes);
  bool addFlags(unsigned int ip, unsigned flag);
  int getFlags(unsigned int ip);
  int getCounterNumber();
  bool Read(int fileFormat, char* fileName);
  bool Save(int fileFormat, char* fileName);
  int RemoveUnlisted(void (*func) (unsigned, unsigned));
  void Print();
  TrafficTableIterator *Sort(int column);
  TrafficDataIterator *getIterator();

private:
  int counterNumber(const char *counter);
  void freeMemory();
  struct ip_entry *findIP(unsigned int ip);
//    int compareEntries(const void* x, const void* y);

  struct ip_entry *First;
  struct ip_entry *Last;
  struct ip_entry *Last_Returned;
  int Entries;
  time_t TimeStamp;
};

class TrafficDataIterator
{
public:
  TrafficDataIterator(struct ip_entry *first);
  int Next();
//    int Previous();
//    void Restart();
  unsigned int getIP();
  unsigned getFlags();
  unsigned long long getUpload(int which);
  unsigned long long getDownload(int which);
  unsigned getCounterNumber();
  char *getCounterName(int which);
private:
  struct ip_entry *Current;
};

class TrafficTableIterator
{
public:
  TrafficTableIterator(struct ip_entry **table, int entries);
   ~TrafficTableIterator();
  int Next();
  unsigned int getIP();
  unsigned getFlags();
  unsigned long long getUpload(int which);
  unsigned long long getDownload(int which);
  unsigned getCounterNumber();
  char *getCounterName(int which);
  unsigned int getNumberOfEntries();
private:
  unsigned int Current;
  struct ip_entry **Table;
  unsigned int Entries;
};
