/* IP-Accounting C++ version
 * program for making statistics of network usage by IP adress
 *  v0.2 Copyright (C) 2004 Jan Wasserbauer <wasik@lbcfree.net>
                                                                                
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
                                                                                
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
                                                                                
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
                                                                                
 */

#include <stdio.h>

#include "config.h"
#include "data.h"
#include "xmlparser.h"
#include "textdatafile.h"
#include "cgioutput.h"

int main(int argc, char *argv[])
{
  /* initialize */
  Config *config = new Config();
  TrafficData *data = new TrafficData();

  int sortByColumn = 0;
  /* read query_string */
  char *qs = getenv("QUERY_STRING");
  if (qs == NULL)
    fprintf(stderr, "QUERY_STRING variable not found.\n");
  else
    sortByColumn = atoi(qs) % ((data->getCounterNumber() - 1) * 2 + 1);

  /* read config file */
  XMLParser *xmlp = new XMLParser();
  xmlp->parseConfigFile(config, CONFIG_FILE);
  delete xmlp;
//  config->Print();

  /* read datafile */
  char df[MAX_LINELEN];
  sprintf(df, "%s/%s", config->DataDir, config->DataFileName);
                                                                                
  if(!data->Read(config->DataFormat, df))
  {
    fprintf(stderr,"Can't read data file %d:%s,", config->DataFormat, df);
    sprintf(df, "%s/%s", config->DataDir, config->OldDataFileName);
    fprintf(stderr," trying old data file %d:%s.\n", config->OldDataFormat, df);
    if(!data->Read(config->DataFormat, df))
    {
        fprintf(stderr,"Can't read old datafile.\n");
        return -1;
    }
  }
                                                                                
//  if (config->Debug)
//    data->Print();

  CGIOutput *cgi = new CGIOutput();
  TrafficTableIterator *it = data->Sort(sortByColumn);

  if (it)
  {
    cgi->PrintPage(config, it, sortByColumn);
    delete it;
  }

  delete cgi;
  delete data;
  delete config;

  return 0;
}
