/* IP-Accounting C++ version 
 * program for making statistics of network usage by IP adress
 *  v0.2 Copyright (C) 2004 Jan Wasserbauer <wasik@lbcfree.net>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include <time.h>

#include "config.h"
#include "data.h"
#include "xmlparser.h"
#include "util.h"
#include "iptables.h"
#include "rrd.h"
#include "xmlout.h"
#include "textdatafile.h"
#include "arpreader.h"
#include "iplist.h"
#include "lbcfreelist.h"

IPTables *ipt = NULL;
Config *config = NULL;
RRD *rrd = NULL;

void RemoveUnlistedAddr(unsigned ip, unsigned flags)
{
      if(ipt)
      { //remove iptables rule
        ipt->Remove(ip, flags);
      }
                                                                                
      if(config->DeleteRRDs)
      { //delete unused RRD file
        rrd->Remove(ip);
      }

}

int main(int argc, char *argv[])
{
  /* initialize */
  config = new Config();
  TrafficData *data = new TrafficData();
  ipt = new IPTables(config);

  /* read config file */
  config->Read(CONFIG_FILE);
  config->Print();

  /* check for correct iptables setting */
  ipt->Check();

  /* read and zero iptables stats */
  time_t timestamp = time(NULL);
  ipt->Read(data);

  /* input ip-lists */
  for(unsigned int i=0; i < config->InputNum; i++)
  {
    switch(config->Input[i].type)
    {
      case INPUT_ARP:
      {
        ARPReader *arp = new ARPReader(config);
        arp->Read(data, config->Input[i].name);
        delete arp;
	break;
      }
	
      case INPUT_LBCFREE:
      {
	LbcFreeList *lbc = new LbcFreeList();
	lbc->Read(data, config->Input[i].name);
	delete lbc;
	break;
      }
	
      case INPUT_LIST:
      {
	IPList *ipl = new IPList();
	ipl->Read(data, config->Input[i].name);
	delete ipl;
	break;
      }
    }
  }

  /* read datafile */
  char df[MAX_LINELEN];
  sprintf(df, "%s/%s", config->DataDir, config->DataFileName);
  
  if(!data->Read(config->DataFormat, df))
  {
    printf("Can't read data file %d:%s,", config->DataFormat, df);
    sprintf(df, "%s/%s", config->DataDir, config->OldDataFileName);
    printf(" trying old data file %d:%s.\n", config->OldDataFormat, df);
    if(!data->Read(config->OldDataFormat, df))
	printf("Can't read old datafile, starting new database.\n");
  }
  
  if(config->Debug)
    data->Print();

  /* clean up entries which are not in the list */
  rrd = new RRD(config);

  if(config->RemoveUnlistedAddresses)
  {
    data->RemoveUnlisted(&RemoveUnlistedAddr);
  }
  
  /* write iptables rules */
  ipt->AddRules(data->getIterator());

  /* save values to rrd database */
  rrd->Save(data->getIterator(), timestamp);
  delete rrd;

  /* write datafile */
  if (!config->TestRun)
  {
    sprintf(df, "%s/%s", config->DataDir, config->DataFileName);
    if(!data->Save(config->DataFormat, df))
	fprintf(stderr,"Can't save datafile %d:%s\n", config->DataFormat, df);
  }

  /* free memory */
  delete ipt;
  delete data;
  delete config;

  return 0;
}
