#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#include "config.h"
#include "data.h"
#include "iptables.h"

IPTables::IPTables(Config * conf)
{
  Conf = conf;
}

int IPTables::findChain(FILE * f, char *chain)
{
  int ret = 0;
  char *s = NULL;
  size_t len = 0;

  while (getline(&s, &len, f) != -1)
  {
    strtok(s, " \t");
    if (strcmp(s, chain) == 0)
    {
      ret = 1;
      break;
    }
  }

  if (s)
    free(s);

  return ret;
}

void IPTables::Check()
{
  if (Conf->TestRun)
    return;

  char ipt[MAX_LINELEN];
  sprintf(ipt, "%s -t mangle -n -L POSTROUTING", Conf->IpTables);
  printf("%s\n", ipt);
  FILE *p = popen(ipt, "r");
  if (!findChain(p, "DATA_IN"))
  {
    pclose(p);
    sprintf(ipt, "%s -t mangle -N DATA_IN", Conf->IpTables);
    system(ipt);
    printf("%s\n", ipt);
    sprintf(ipt, "%s -t mangle -A POSTROUTING -j DATA_IN", Conf->IpTables);
    system(ipt);
    printf("%s\n", ipt);
  }
  else
    pclose(p);

  sprintf(ipt, "%s -t mangle -n -L PREROUTING", Conf->IpTables);
  p = popen(ipt, "r");
  printf("%s\n", ipt);
  if (!findChain(p, "DATA_OUT"))
  {
    pclose(p);
    sprintf(ipt, "%s -t mangle -N DATA_OUT", Conf->IpTables);
    printf("%s\n", ipt);
    system(ipt);
    sprintf(ipt, "%s -t mangle -A PREROUTING -j DATA_OUT", Conf->IpTables);
    printf("%s\n", ipt);
    system(ipt);
  }
  else
    pclose(p);
}


int IPTables::parseLine(int io, char *s, unsigned int *ip,
                        unsigned long long *bytes)
{
//XXX rewrite
  char *t;
  s = skip_whitespace(s);       //mezery na zacatku radku
  if (!*s)
    return 0;

  s = skip_token(s);            //1 sloupec pkts
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;

  t = skip_token(s);            //2 sloupec bytes
  if (!*t)
    return 0;
  *t = 0;
  *bytes = strtoull(s, NULL, 10);

  s = skip_whitespace(t + 1);
  if (!*s)
    return 0;
  t = skip_token(s);            //3 sloupec target
  if (!*t)
    return 0;
  *t = 0;
  if (strcmp(s, "RETURN") != 0)
    return 0;

  s = skip_whitespace(t + 1);
  if (!*s)
    return 0;
  s = skip_token(s);            //4 sloupec prot
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;
  s = skip_token(s);            //5 sloupec opt
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;
  s = skip_token(s);            //6 sloupec in
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;
  s = skip_token(s);            //7 sloupec out
  if (!*s)
    return 0;

  s = skip_whitespace(s);
  if (!*s)
    return 0;
  if (io == F_IPT_IN)
  {
    s = skip_token(s);          //8 sloupec source
    if (!*s)
      return 0;

    s = skip_whitespace(s);
    if (!*s)
      return 0;
  }

  t = skip_token(s);            //8 nebo 9 sloupec source nebo destination
  if (!*t)
    return 0;
  *t = 0;
  *ip = parse_ip(s);
  if (!*ip)
    return 0;

  return 1;
}

void IPTables::readData(TrafficData * data, int flags)
{
  char ipt[MAX_LINELEN];
  char *chain;
  char zero[4];

  *zero = 0;
  if (!Conf->TestRun)
    strcpy(zero, "-Z");

  if (flags == F_IPT_IN)
  {
    chain = "DATA_IN";
  }
  else
  {
    chain = "DATA_OUT";
  }
  sprintf(ipt, "%s -t mangle -L %s -v -x -n %s", Conf->IpTables, chain, zero);
  printf("%s\n", ipt);
  FILE *p = popen(ipt, "r");

  char *s = NULL;
  size_t len = 0;
  if (!p)
  {
    fprintf(stderr, "Nelze spustit %s: %s\n", ipt, strerror(errno));
    exit(-1);
  }

  unsigned ip;
  unsigned long long bytes;

  while (getline(&s, &len, p) != -1)
  {
    if (parseLine(flags, s, &ip, &bytes))
    {
      int fl = data->getFlags(ip);
      if (fl != -1 && (fl & flags))
      {
	fprintf(stderr, "IP address %s is twice in iptables chain %s\n", string_ip(ip), chain);
	continue;
      }
 
      if (flags & F_IPT_IN)
        data->setDownload(ip, 0, bytes);
      else
        data->setUpload(ip, 0, bytes);

      data->addFlags(ip, flags);
    }
  }

  pclose(p);
  if (s)
    free(s);
}

void IPTables::Read(TrafficData * data)
{
  readData(data, F_IPT_IN);
  readData(data, F_IPT_OUT);
}

void IPTables::AddRules(TrafficDataIterator * it)
{
  unsigned int ip, flags;
  char cmd[MAX_LINELEN];

  if (!it)
    return;

  do
  {
    ip = it->getIP();
    flags = it->getFlags();
//                if(flags & F_PROC_ARP) //aktivni ip adresa
    {
      if (!(flags & F_IPT_IN))
      {
        sprintf(cmd, "%s -t mangle -A DATA_IN -d %s -j RETURN",
                Conf->IpTables, string_ip(ip));
        printf("%s\n", cmd);
        if (!Conf->TestRun)
          system(cmd);
      }
      if (!(flags & F_IPT_OUT))
      {
        sprintf(cmd, "%s -t mangle -A DATA_OUT -s %s -j RETURN",
                Conf->IpTables, string_ip(ip));
        printf("%s\n", cmd);
        if (!Conf->TestRun)
          system(cmd);
      }
    }
  }
  while (it->Next());
}

void IPTables::Remove(unsigned ip, unsigned flags)
{
  char cmd[MAX_LINELEN];
  if (flags & F_IPT_IN)
  {
    sprintf(cmd, "%s -t mangle -D DATA_IN -d %s -j RETURN",
	    Conf->IpTables, string_ip(ip));
    printf("%s\n", cmd);
    if (!Conf->TestRun)
      system(cmd);
  }
  if (flags & F_IPT_OUT)
  {
    sprintf(cmd, "%s -t mangle -D DATA_OUT -s %s -j RETURN",
	    Conf->IpTables, string_ip(ip));
    printf("%s\n", cmd);
    if (!Conf->TestRun)
      system(cmd);
  }
}

