#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "config.h"
#include "data.h"
#include "textdatafile.h"

char *DataFileList[] =
  { "-daily.txt", "-weekly.txt", "-monthly.txt",
"-yearly.txt", ".txt", 0 };

int TextDataFile::parseDataFileLine(char *s, unsigned *ip,
                                    unsigned long long *bytes_up,
                                    unsigned long long *bytes_down)
{
  char *t;
  t = s;

  while (*t != 0 && *t != '-')
    t++;
  if (!t)
    return 0;
  *t = 0;
  *ip = parse_ip(s);
  t++;
  s = t;
  while (*t != 0 && *t != '-')
    t++;
  if (!t)
    return 0;
  *bytes_up = strtoull(s, (char **) NULL, 10);
  t++;
  s = t;
  while (*t != 0 && *t != '-' && *t != '\\')
    t++;
  if (t == s)
    return 0;
  *bytes_down = strtoull(s, (char **) NULL, 10);
  return 1;
}


int TextDataFile::readDataFile(TrafficData * data, char *datafile, int which)
{
  char *s = NULL;
  size_t len = 0;
  FILE *f = fopen(datafile, "rt");
  if (!f)
  {
    if (errno == ENOENT)
      return 0;
    fprintf(stderr, "Nelze otevrit %s: %s\n", datafile, strerror(errno));
    return 0;
  }


  unsigned ip;
  unsigned long long bytes_up, bytes_down;


  while (getline(&s, &len, f) != -1)
  {
    if (parseDataFileLine(s, &ip, &bytes_up, &bytes_down))
    {
      data->setDownload(ip, which, bytes_down);
      data->setUpload(ip, which, bytes_up);
    }
  }


  fclose(f);
  if (s)
    free(s);

  return 1;
}

int TextDataFile::saveDataFile(TrafficData * data, char *datafile, int which)
{
  FILE *f = fopen(datafile, "wt");
  if (!f)
  {
    fprintf(stderr, "Nelze otevrit %s: %s\n", datafile, strerror(errno));
    return 0;
  }

  TrafficDataIterator *it = data->getIterator();

  do
  {
    fprintf(f, "%s-%llu-%llu\n", string_ip(it->getIP()),
            it->getUpload(which) + it->getUpload(0),
            it->getDownload(which) + it->getDownload(0));
  }
  while (it->Next());

  fclose(f);
  return 1;
}

void TextDataFile::expireDataFile(char *datafile)
{
  struct stat buf;

  if (stat(datafile, &buf) != 0)
  {
    if (errno != ENOENT)
      fprintf(stderr, "stat %s: %s\n", datafile, strerror(errno));
    return;
  }

  time_t tt = time(NULL);
  struct tm t_act;
  struct tm t_df;
  localtime_r(&tt, &t_act);
  localtime_r(&buf.st_mtime, &t_df);
  int del = 0;

  if (strstr(datafile, "daily") != NULL)
  {
    if (!
        (t_df.tm_mday == t_act.tm_mday && t_df.tm_mon == t_act.tm_mon
         && t_df.tm_year == t_act.tm_year))
      del = 1;
  }
  else if (strstr(datafile, "weekly") != NULL)
  {
    int cz_week = t_act.tm_wday - 1;
    if (cz_week == -1)
      cz_week = 6;
    int start_of_week = t_act.tm_yday - cz_week;
    if (t_act.tm_year == t_df.tm_year)
    {
      if (t_df.tm_yday < start_of_week)
        del = 1;
    }
    else if (start_of_week < 0 && t_df.tm_year == t_act.tm_year - 1)
    {
      start_of_week = 364 + start_of_week;
      if (t_df.tm_year % 4 == 0)
        start_of_week++;
      if (t_df.tm_yday < start_of_week)
        del = 1;
    }
    else
      del = 1;
  }
  else if (strstr(datafile, "monthly") != NULL)
  {
    if (!(t_df.tm_year == t_act.tm_year && t_df.tm_mon == t_act.tm_mon))
      del = 1;
  }
  else if (strstr(datafile, "yearly") != NULL)
  {
    if (t_df.tm_year != t_act.tm_year)
      del = 1;
  }

  if (del)
  {
    printf("%s has expired, removing\n", datafile);
    if (unlink(datafile) != 0)
    {
      fprintf(stderr, "Can't remove %s: %s\n", datafile, strerror(errno));
    }
  }
}

int TextDataFile::Read(TrafficData * data, char* fileName)
{
  int i = 0;
  char datafile[MAX_LINELEN];
  int ret = 0;
  while (DataFileList[i])
  {
    sprintf(datafile, "%s%s", fileName, DataFileList[i]);

    expireDataFile(datafile);
    if (readDataFile(data, datafile, i + 1))
      ret = 1;
    i++;
  }

  return ret;
}

int TextDataFile::Save(TrafficData * data, char* fileName)
{
  int i = 0;
  char datafile[MAX_LINELEN];
  while (DataFileList[i])
  {
    sprintf(datafile, "%s%s", fileName, DataFileList[i]);

    saveDataFile(data, datafile, i + 1);
    i++;
  }

  return 1;

}
